<?php

/**
 * @file
 * Helper functions file for Entity Documentation Confluence module.
 */

/**
 * Exports the documentation to Confluence.
 *
 * Handles the retrieve, create and delete actions between drupal and
 * Confluence restful API.
 * This function runs, only if the required administration settings saved
 * successfully.
 * Consist of a series of steps taken in order to create or delete a Confluence
 * page.
 * First of all checks if the requested confluence page name does exist. If it
 * does not exist, then the function creates this page under the entity parent
 * page (Parent page for "node" entities). Otherwise, first deletes this page
 * and then creates it again.
 * But if the entity parent page (Parent page for "node" entities) does not
 * exist, first creates the entity parent page and then, under this page creates
 * the bundle documentation page.
 *
 * @param string $entity
 *   The entity type.
 * @param string $bundle
 *   The bundle name.
 * @param array $variable
 *   The variable array.
 *
 * @return array|int
 *   Returns the http response data if something went wrong otherwise returns
 *   the status code '200'.
 */
function entity_documentation_confluence_export_documentation($entity, $bundle, array $variable) {

  // Make a retrieve request to find out what to do next.
  $response = entity_documentation_confluence_create_request('retrieve', $entity, $variable, $bundle);

  if (is_numeric($response)) {
    // This means that something went wrong and the response code is not the
    // expected one.
    return $response;
  }
  else {
    if (count($response->results) == '0') {
      if ($variable[$entity . '_parent_id'] != NULL) {
        // There is a saved value for the entity parent page in the variable.
        // Check if the saved entity parent page does exist in confluence pages
        // and has the same id with the one that is stored in variable.
        $response = entity_documentation_confluence_create_request('retrieve', $entity, $variable, $bundle, $variable[$entity . '_parent_id']);

        if (is_numeric($response)) {
          // Entity parent page id is invalid.
          $variable = _entity_documentation_confluence_update_variable('update', $entity, $variable);
          $response = entity_documentation_confluence_create_request('add', $entity, $variable, $bundle, $variable['parent_id']);
          _entity_documentation_confluence_update_variable('create', $entity, $variable, $response->id);
        }
        else {
          // Entity parent page id does exist.
          $response = entity_documentation_confluence_create_request('add', $entity, $variable, $bundle, $variable[$entity . '_parent_id']);
        }
      }
      else {
        // There is not a saved value for the entity parent page in the variable
        // so we have to check if there is one in the confluence pages.
        $response = entity_documentation_confluence_create_request('retrieve', $entity, $variable, $entity);

        if (is_numeric($response)) {
          return $response;
        }
        else {
          if (count($response->results) == '1') {
            // Entity parent page already exist but for some reason the value is
            // not saved in the variable.
            $variable = _entity_documentation_confluence_update_variable('create', $entity, $variable, $response->results[0]->id);
            $response = entity_documentation_confluence_create_request('add', $entity, $variable, $bundle, $variable[$entity . '_parent_id']);
          }
          else {
            // Variable is not set so create the entity parent page.
            $response = entity_documentation_confluence_create_request('add', $entity, $variable, $bundle, $variable['parent_id']);
            $variable = _entity_documentation_confluence_update_variable('create', $entity, $variable, $response->id);
            $response = entity_documentation_confluence_create_request('add', $entity, $variable, $bundle, $variable[$entity . '_parent_id']);
          }
        }
      }
    }
    elseif (count($response->results) == '1') {
      if (isset($variable[$entity . '_parent_id'])) {
        // First delete the existing page and then create the updated one..
        entity_documentation_confluence_create_request('delete', $entity, $variable, $bundle, $response->results[0]->id);
        $exist = entity_documentation_confluence_create_request('retrieve', $entity, $variable, $bundle, $variable[$entity . '_parent_id']);

        if (is_numeric($exist)) {
          // Update variable.
          $variable = _entity_documentation_confluence_update_variable('update', $entity, $variable);
          $response = entity_documentation_confluence_create_request('add', $entity, $variable, $bundle, $variable['parent_id']);
          $variable = _entity_documentation_confluence_update_variable('create', $entity, $variable, $response->id);
          $response = entity_documentation_confluence_create_request('add', $entity, $variable, $bundle, $variable['parent_id']);
        }
        else {
          $response = entity_documentation_confluence_create_request('add', $entity, $variable, $bundle, $variable[$entity . '_parent_id']);
        }
      }
    }
    // Return the proper http response code.
    if (is_numeric($response)) {
      return $response;
    }
    else {
      return '200';
    }
  }
}

/**
 * Function that handles the entity parent id variable value.
 *
 * This function is responsible to create or update the module's entity parent
 * id variable. The 'create' action runs if entity parent id does not exist in
 * the available array. In addition the 'update' action runs only if the saved
 * value must change.
 *
 * @param string $action
 *   The action to perform to the variable. Can be 'create' or 'update'.
 * @param string $entity
 *   The entity type.
 * @param array $variable
 *   The variable values.
 * @param string $parent_id
 *   The parent page id. It is required only for the 'create' action.
 *
 * @return array
 *   The altered $variable.
 */
function _entity_documentation_confluence_update_variable($action, $entity, array $variable, $parent_id = '') {

  switch ($action) {
    case 'create':
      $variable[$entity . '_parent_id'] = $parent_id;
      $serialized = serialize($variable);
      variable_set('ed_confluence_api_values', $serialized);
      break;

    case 'update':
      unset($variable[$entity . '_parent_id']);
      $serialized = serialize($variable);
      variable_set('ed_confluence_api_values', $serialized);
      break;

  }

  return $variable;
}

/**
 * Function wrapper that makes the http request.
 *
 * @param string $method
 *   The request method can be 'retrieve', 'add' or 'delete'.
 * @param string $entity
 *   The entity type.
 * @param array $variable
 *   The variable values.
 * @param string $bundle
 *   The bundle name.
 * @param int|string $page_id
 *   The confluence page id is optional. If entered means that the request need
 *   to handle a specific confluence page.
 *
 * @return array|int
 *   Returns the decoded response data array or the response code status.
 */
function entity_documentation_confluence_create_request($method, $entity, array $variable, $bundle, $page_id = '') {

  // Get the http request arguments and make the request.
  $arguments = entity_documentation_confluence_get_http_arguments($method, $entity, $variable, $bundle, $page_id);
  $response = drupal_http_request($arguments['uri'], $arguments['options']);

  if ($response->code == '400') {
    return $response->code;
  }
  elseif ($response->code != '200' && $response->code != '204') {
    watchdog('entity_documentation_confluence', 'Http request failed. Response data @data',
      array('@data' => $response->data), WATCHDOG_ERROR, NULL);
    return $response->code;
  }
  else {
    $decoded = json_decode($response->data);
    return $decoded;
  }
}

/**
 * Function that generates the http request arguments.
 *
 * Based on the selected $method this function uses the module's form settings
 * to built the http arguments for the request. Builds either Authorized
 * requests or non Authorized requests automatically. Also uses json format
 * for the request body value.
 *
 * @param string $method
 *   The request method can be 'retrieve', 'add' or 'delete'.
 * @param string $entity
 *   The entity type.
 * @param array $variable
 *   The variable array.
 * @param string $bundle
 *   The bundle name.
 * @param int|string $page_id
 *   The confluence page id is optional. If entered means that the request need
 *   to handle a specific confluence page. Is required for the delete request as
 *   defined in confluence api documentation.
 *
 * @return array
 *   Returns the http request arguments as an array.
 */
function entity_documentation_confluence_get_http_arguments($method, $entity, array $variable, $bundle, $page_id = '') {

  switch ($method) {
    case 'retrieve':
      // The retrieve request for a confluence page can be either by a page id
      // or by a page title. This separation is depending on the $page_id value.
      if ($page_id != '') {
        $uri = url($variable['url'] . '/rest/api/content/' . $page_id . '?type=page&spaceKey=' . $variable['space_key'] . '&expand=ancestors');
      }
      else {
        $title = _entity_documentation_confluence_create_page_title($bundle);
        $title = preg_replace('/ /', '+', $title);
        $uri = url($variable['url'] . '/rest/api/content?type=page&spaceKey=' . $variable['space_key'] . '&title=' . $title . '&expand=ancestors');
      }

      $options = array(
        'method' => 'GET',
        'headers' => array(),
      );

      if (isset($variable['encoded'])) {
        // Request Headers.
        $options['headers'] = array(
          'Authorization' => 'Basic ' . $variable['encoded'],
          'os_authType' => 'Basic',
        );
      }
      break;

    case 'add':
      $uri = url($variable['url'] . '/rest/api/content');
      // Get json data.
      $json = _entity_documentation_confluence_get_json_data($entity, $bundle, $variable);

      $options = array(
        'method' => 'POST',
        'data' => $json,
        'headers' => array(),
      );

      // Header without Authorization.
      $options['headers'] = array(
        'Accept' => 'application/json',
        'Content-Type' => 'application/json',
        'dataType' => 'json',
        'spaceKey' => $variable['space_key'],
      );

      // Request Headers with Authorization.
      if (isset($variable['encoded'])) {
        $options['headers']['Authorization'] = 'Basic ' . $variable['encoded'];
        $options['headers']['os_authType'] = 'Basic';
      }
      break;

    case 'delete':
      // The delete request requires a $page_id value.
      $options = array(
        'method' => 'DELETE',
        'headers' => array(),
      );

      // Header without Authorization.
      $options['headers'] = array(
        'Content-Type' => 'application/json',
        'spaceKey' => $variable['space_key'],
      );

      // Request Headers with Authorization.
      if (isset($variable['encoded'])) {
        $options['headers']['Authorization'] = 'Basic ' . $variable['encoded'];
        $options['headers']['os_authType'] = 'Basic';
      }

      $uri = url($variable['url'] . '/rest/api/content/' . $page_id);
      break;

    default:
      // The selected method is not valid.
      watchdog('entity_documentation_confluence', 'Invalid method @method',
        array('@method' => $method), WATCHDOG_ERROR, NULL);
      return FALSE;
  }

  // Arguments to return.
  $arguments = array(
    'uri' => $uri,
    'options' => $options,
  );

  // Creating hook to alter the request arguments will call all modules
  // implementing hook_edc_http_arguments_alter.
  drupal_alter('edc_http_arguments_alter', $arguments);

  return $arguments;
}

/**
 * Creates the json data for the request body.
 *
 * @param string $entity
 *   The entity type.
 * @param string $bundle
 *   The bundle name.
 * @param array $variable
 *   The variable array.
 *
 * @return string
 *   Returns the json array.
 */
function _entity_documentation_confluence_get_json_data($entity, $bundle, array $variable) {

  if (isset($variable[$entity . '_parent_id'])) {
    // Generate the confluence page body value of a bundle page.
    $new_bundle_name = _entity_documentation_confluence_create_page_title($bundle);
    // Get documentation array.
    module_load_include('inc', 'entity_documentation', 'includes/entity_documentation.functions');
    $documentation_array = ed_bundle_array($entity, $bundle);

    // Get the content we want to convert into Confluence page by passing the
    // required values.
    $html = theme(
      'entity_documentation_confluence',
      array(
        'description' => $documentation_array[$entity][$bundle]['params']['description'],
        'properties' => $documentation_array[$entity][$bundle]['params']['properties'],
        'field_columns' => $documentation_array[$entity][$bundle]['field_columns'],
        'fields' => $documentation_array[$entity][$bundle]['fields'],
      )
    );

    // Built the json array for the request.
    $json = array(
      'type' => 'page',
      'title' => $new_bundle_name,
      'space' => array(
        'key' => $variable['space_key'],
      ),
      'ancestors' => array(
        array('id' => $variable[$entity . '_parent_id']),
      ),
      'body' => array(
        'storage' => array(
          'value' => $html,
          'representation' => 'storage',
        ),
      ),
    );

    $json = json_encode($json);
  }
  else {
    // Generate the entity parent page title.
    $entity_name = _entity_documentation_confluence_create_page_title($entity);

    // Built the json array for the request.Body value is null.
    $json = array(
      'type' => 'page',
      'title' => $entity_name,
      'space' => array(
        'key' => $variable['space_key'],
      ),
      'ancestors' => array(
        array('id' => $variable['parent_id']),
      ),
      'body' => array(
        'storage' => array(
          'value' => '',
          'representation' => 'storage',
        ),
      ),
    );

    $json = json_encode($json);
  }

  return $json;
}

/**
 * Create human readable page name from machine name.
 *
 * @param string $name
 *   The machine name.
 *
 * @return mixed|string
 *   Returns the human readable page name.
 */
function _entity_documentation_confluence_create_page_title($name) {
  $page_name = preg_replace('/_/', ' ', $name);
  $page_name = ucwords($page_name);

  $page_name = $page_name . '[' . $name . ']';
  return $page_name;
}
